import asyncio
import websockets
import socket
import threading
import json
import os

# Файлы и директории
users_file = 'users.json'
servers_dir = 'servers'
clients = {}  # TCP clients
active_users = {}  # WebSocket clients
websocket_clients = {}

# Убедитесь, что директория для серверов существует
os.makedirs(servers_dir, exist_ok=True)

# Загрузка и сохранение пользователей
def load_users():
    if os.path.exists(users_file):
        with open(users_file, 'r', encoding='utf-8') as f:
            return json.load(f)
    return {}

def save_users(users):
    with open(users_file, 'w', encoding='utf-8') as f:
        json.dump(users, f, ensure_ascii=False, indent=4)

# Загрузка и сохранение серверов
def load_servers():
    servers = {}
    for filename in os.listdir(servers_dir):
        if filename.endswith('.json'):
            server_name = filename[:-5]
            with open(os.path.join(servers_dir, filename), 'r', encoding='utf-8') as f:
                servers[server_name] = json.load(f)
    return servers

def save_server(server_name, server_data):
    with open(os.path.join(servers_dir, f'{server_name}.json'), 'w', encoding='utf-8') as f:
        json.dump(server_data, f, ensure_ascii=False, indent=4)

# Глобальные переменные
users = load_users()
servers = load_servers()

# Убедитесь, что существует сервер по умолчанию
default_server = 'general'
if default_server not in servers:
    servers[default_server] = []
    save_server(default_server, servers[default_server])

# Отправка сообщения всем клиентам на сервере
async def broadcast_message(message, server_name, sender=None):
    if server_name not in servers:
        return

    tasks = []
    server_members = set(servers[server_name])  # Используйте множество для предотвращения дублирования

    # Отправка сообщений WebSocket клиентам
    for username in server_members:
        ws = active_users.get(username)
        if ws and ws != sender:
            try:
                tasks.append(ws.send(message))  # Убедитесь, что сообщение - строка
            except Exception as e:
                print(f"Error sending message to WebSocket client: {e}")

    # Отправка сообщений TCP клиентам
    tcp_clients = clients.get(server_name, [])
    for client in tcp_clients:
        try:
            client.send(message.encode('utf-8'))
        except Exception as e:
            print(f"Error sending message to TCP client: {e}")

    if tasks:
        await asyncio.gather(*tasks)

# Список доступных команд
commands = {
    "/login": "Usage: /login <username> <password> - Log in with your username and password.",
    "/register": "Usage: /register <username> <password> - Register a new account.",
    "/create_server": "Usage: /create_server <server_name> - Create a new server.",
    "/join_server": "Usage: /join_server <server_name> - Join an existing server.",
    "/list_servers": "Usage: /list_servers - List all available servers.",
    "/members": "Usage: /members - List all members in the current server.",
    "/help": "Usage: /help - Show this help message."
}

# Обработка WebSocket соединений
async def handle_websocket(websocket, path):
    print("WebSocket client connected")
    username = None
    user_server = default_server  # Сервер по умолчанию для новых пользователей

    try:
        async for message in websocket:
            print(f"WebSocket Message: {message}")

            if message.startswith("/"):
                if message.startswith("/login"):
                    parts = message.split(" ", 2)
                    if len(parts) != 3:
                        await websocket.send("Usage: /login <username> <password>")
                        continue
                    _, username, password = parts
                    if users.get(username) == password:
                        if username in active_users:
                            await websocket.send("This username is already in use.")
                        else:
                            active_users[username] = websocket
                            await websocket.send("Login successful.")
                            await websocket.send(f"Available servers: {', '.join(servers.keys())}")
                            await websocket.send(f"Select a server using /join_server <server_name>.")
                    else:
                        await websocket.send("Invalid username or password.")
                elif message.startswith("/register"):
                    parts = message.split(" ", 2)
                    if len(parts) != 3:
                        await websocket.send("Usage: /register <username> <password>")
                        continue
                    _, username, password = parts
                    if username in users:
                        await websocket.send("Username already taken.")
                    else:
                        users[username] = password
                        save_users(users)
                        await websocket.send("Registration successful. Please log in.")
                elif message.startswith("/create_server"):
                    parts = message.split(" ", 1)
                    if len(parts) != 2:
                        await websocket.send("Usage: /create_server <server_name>")
                        continue
                    _, server_name = parts
                    if server_name in servers:
                        await websocket.send("Server already exists.")
                    else:
                        servers[server_name] = []
                        save_server(server_name, servers[server_name])
                        await websocket.send(f"Server '{server_name}' created successfully.")
                elif message.startswith("/join_server"):
                    parts = message.split(" ", 1)
                    if len(parts) != 2:
                        await websocket.send("Usage: /join_server <server_name>")
                        continue
                    _, server_name = parts
                    if server_name not in servers:
                        await websocket.send("Server does not exist.")
                    else:
                        if username:
                            if user_server and username in servers.get(user_server, []):
                                servers[user_server].remove(username)
                                save_server(user_server, servers[user_server])
                        user_server = server_name
                        if username and username not in servers.get(server_name, []):
                            servers[server_name].append(username)
                            save_server(server_name, servers[server_name])
                        await websocket.send(f"Joined server '{server_name}' successfully.")
                elif message.startswith("/list_servers"):
                    await websocket.send(f"Servers: {', '.join(servers.keys())}")
                elif message.startswith("/members"):
                    if username and user_server:
                        members = servers.get(user_server, [])
                        await websocket.send(f"Members in '{user_server}': {', '.join(members)}")
                    else:
                        await websocket.send("Please log in and join a server first.")
                elif message.startswith("/help"):
                    help_message = "\n".join([f"{cmd}: {desc}" for cmd, desc in commands.items()])
                    await websocket.send(help_message)
                else:
                    await websocket.send("Unknown command.")
            else:
                if username and user_server:
                    full_message = f"{username}: {message}"
                    await broadcast_message(full_message, user_server, websocket)
                else:
                    await websocket.send("Please log in and join a server first.")

    except Exception as e:
        print(f"WebSocket error: {e}")
    finally:
        if username:
            del active_users[username]
            if user_server:
                if username in servers.get(user_server, []):
                    servers[user_server].remove(username)
                    save_server(user_server, servers[user_server])
                await broadcast_message(f"{username} has left the server.", user_server)
        print("WebSocket client disconnected")

# Обработка TCP соединений
def handle_client(client_socket, client_address):
    logged_in_user = None
    user_server = default_server  # Сервер по умолчанию для новых пользователей

    while not logged_in_user:
        client_socket.send("Enter command (/login /register): ".encode('utf-8'))
        command = client_socket.recv(1024).decode('utf-8').strip()

        if command.startswith("/register"):
            parts = command.split(" ", 2)
            if len(parts) != 3:
                client_socket.send("Usage: /register <username> <password>\n".encode('utf-8'))
                continue
            _, username, password = parts
            if username in users:
                client_socket.send("Username already taken. Try another.\n".encode('utf-8'))
                continue
            users[username] = password
            save_users(users)
            client_socket.send("Registration successful. Please log in.\n".encode('utf-8'))

        elif command.startswith("/login"):
            parts = command.split(" ", 2)
            if len(parts) != 3:
                client_socket.send("Usage: /login <username> <password>\n".encode('utf-8'))
                continue
            _, username, password = parts
            if users.get(username) == password:
                if username in active_users:
                    client_socket.send("This username is already in use.\n".encode('utf-8'))
                else:
                    logged_in_user = username
                    active_users[username] = client_socket
                    client_socket.send("Login successful.\n".encode('utf-8'))
                    client_socket.send(f"Available servers: {', '.join(servers.keys())}\n".encode('utf-8'))
                    client_socket.send("Select a server using /join_server <server_name>.\n".encode('utf-8'))
            else:
                client_socket.send("Invalid username or password.\n".encode('utf-8'))

    try:
        while True:
            message = client_socket.recv(1024).decode('utf-8').strip()
            if not message:
                break

            if message.startswith("/"):
                if message.startswith("/create_server"):
                    parts = message.split(" ", 1)
                    if len(parts) != 2:
                        client_socket.send("Usage: /create_server <server_name>\n".encode('utf-8'))
                        continue
                    _, server_name = parts
                    if server_name in servers:
                        client_socket.send("Server already exists.\n".encode('utf-8'))
                    else:
                        servers[server_name] = []
                        save_server(server_name, servers[server_name])
                        client_socket.send(f"Server '{server_name}' created successfully.\n".encode('utf-8'))
                elif message.startswith("/join_server"):
                    parts = message.split(" ", 1)
                    if len(parts) != 2:
                        client_socket.send("Usage: /join_server <server_name>\n".encode('utf-8'))
                        continue
                    _, server_name = parts
                    if server_name not in servers:
                        client_socket.send("Server does not exist.\n".encode('utf-8'))
                    else:
                        if user_server and logged_in_user in servers.get(user_server, []):
                            servers[user_server].remove(logged_in_user)
                            save_server(user_server, servers[user_server])
                        user_server = server_name
                        if logged_in_user and logged_in_user not in servers.get(server_name, []):
                            servers[server_name].append(logged_in_user)
                            save_server(server_name, servers[server_name])
                        if server_name not in clients:
                            clients[server_name] = []
                        if client_socket not in clients[server_name]:
                            clients[server_name].append(client_socket)
                        client_socket.send(f"Joined server '{server_name}' successfully.\n".encode('utf-8'))
                elif message.startswith("/list_servers"):
                    client_socket.send(f"Servers: {', '.join(servers.keys())}\n".encode('utf-8'))
                elif message.startswith("/members"):
                    if logged_in_user and user_server:
                        members = servers.get(user_server, [])
                        client_socket.send(f"Members in '{user_server}': {', '.join(members)}\n".encode('utf-8'))
                    else:
                        client_socket.send("Please log in and join a server first.\n".encode('utf-8'))
                elif message.startswith("/help"):
                    help_message = "\n".join([f"{cmd}: {desc}" for cmd, desc in commands.items()])
                    client_socket.send(f"{help_message}\n".encode('utf-8'))
                else:
                    client_socket.send("Unknown command.\n".encode('utf-8'))
            else:
                if logged_in_user and user_server:
                    full_message = f"{logged_in_user}: {message}"
                    asyncio.run(broadcast_message(full_message, user_server))
    except Exception as e:
        print(f"TCP Client error: {e}")
    finally:
        if logged_in_user:
            del active_users[logged_in_user]
            if user_server:
                if logged_in_user in servers.get(user_server, []):
                    servers[user_server].remove(logged_in_user)
                    save_server(user_server, servers[user_server])
                clients.get(user_server, []).remove(client_socket)
                asyncio.run(broadcast_message(f"{logged_in_user} has left the server.", user_server))
        client_socket.close()

# Запуск TCP сервера
def start_tcp_server(host='0.0.0.0', port=0000):
    server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server_socket.bind((host, port))
    server_socket.listen(5)
    print(f"TCP server started on {host}:{port}")

    while True:
        client_socket, client_address = server_socket.accept()
        print(f"TCP client connected from {client_address}")
        client_thread = threading.Thread(target=handle_client, args=(client_socket, client_address))
        client_thread.start()

# Запуск WebSocket сервера
async def start_websocket_server(host='0.0.0.0', port=0000):
    async with websockets.serve(handle_websocket, host, port):
        print(f"WebSocket server started on {host}:{port}")
        await asyncio.Future()  # Run forever

# Запуск серверов
if __name__ == "__main__":
    threading.Thread(target=start_tcp_server).start()
    asyncio.run(start_websocket_server())
